<?php

namespace Drupal\paragraph_locator\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for paragraph deletion tasks.
 */
class ParagraphLocator extends ControllerBase {

  /**
   * The messenger object.
   *
   * @var Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a new messenger object.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(MessengerInterface $messenger) {
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger')
    );
  }

  /**
   * Builds the response.
   */
  public function deleteEntries(Request $request) {
    try {
      $paragraph_ids = $request->get('data');
      if (!empty($paragraph_ids)) {
        foreach ($paragraph_ids as $paragraph_id) {
          $paragraph_data = $this->entityTypeManager()->getStorage('paragraph')->load($paragraph_id['value']);
          if ($paragraph_data) {
            $data = [
              'type' => $paragraph_data->parent_type->value,
              'id' => $paragraph_data->parent_id->value,
            ];
            $entity = $this->entityTypeManager()->getStorage($data['type'])->load($data['id']);
            $field_values = $entity->{$paragraph_id['parent']}->getValue();
            $id = $paragraph_id['value'];
            $new_value = array_filter($field_values, function ($item) use ($id) {
              return $item['target_id'] !== $id;
            });
            $entity->{$paragraph_id['parent']}->setValue($new_value);
            $entity->save();
            $paragraph_data->delete();
            $this->messenger->addMessage($this->t('Paragraph ID @pid deleted.', ['@pid' => $id]));
          }
        }
        $response = [
          'status' => "success",
          'message' => $this->t('Paragraph deleted successfully'),
        ];
      }
    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('An unexpected error occured while deleting paragraph(s): @message', ['@message' => $e->getMessage()]));
      $response = [
        'status' => "error",
        'message' => $this->t('An error occurred: @message', ['@message' => $e->getMessage()]),
      ];
    }

    return new JsonResponse($response);
  }

  /**
   * Function to recusively find the parent of the paragraph type.
   *
   * @param string $id
   *   Stores the paragraph id.
   */
  public function paragraphParent($id) {
    $paragraph = $this->entityTypeManager()->getStorage('paragraph')->load($id);
    if ($paragraph->parent_type->value == 'paragraph') {
      return $this->paragraphParent($paragraph->parent_id->value);
    }
    return [
      'type' => $paragraph->parent_type->value,
      'id' => $paragraph->parent_id->value,
    ];
  }

}
