<?php

namespace Drupal\paragraph_locator;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class to create a custom service including helper functions.
 */
class ParagraphLocatorHelper {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructor function.
   *
   * @param Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Helper function to recursively load the parent entity.
   *
   * @param string $id
   *   Accepts the id of an entity.
   * @return mixed
   *   Returns the parent entity if any otherwise NULL.
   */
  public function getParentEntity($id) {
    // Check if the entity has a parent.
    if ($id) {
      // Get the parent entity.
      $current_paragraph = $this->entityTypeManager->getStorage('paragraph')->load($id);

      // If the parent entity type is not a node,
      // continue to the next parent recursively.
      if ($current_paragraph->parent_type && $current_paragraph->parent_type->value !== 'node') {
        return $this->getParentEntity($current_paragraph->parent_id->value);
      }
      return $current_paragraph;
    }

    return NULL;
  }

}
